/**
 * amlet_analysis.h
 *
 * AMLET (Another Mixed Logit Estimation Tool)
 * Copyright (C) 2002-2004 Fabian Bastin <fbas@math.fundp.ac.be>
 *
 * Licensed under the Open Software License version 2.1
 */

#ifndef __AMLET_ANALYSIS_H__
#define __AMLET_ANALYSIS_H__

#include <amlet.h>

#include <amlet_driver.h>

#include <oratio/cblas.h>
#include <oratio/f2c.h>

#include <amlet/amlet.h>

/**
 * Analysis of AMLET results
 */

#include <driver.h>

#ifdef __cplusplus
extern "C" {
#endif

  int count_parameters(int law);

  /**
   * Compute the log-likelihood value when all parameters are set to
   * zero and when only alternative specific constants are considered.
   *
   * @param ismixed is an integer equal to 0 if we face a classical
   *    multinomial model, another value elsewhere.
   */
  void special_likelihood_functions(int ismixed, double *ll0, double *llc,
				    dcl_data *md, driver *dr,
				    double *work);

  /**
   * Statistical tests
   */

  /**
   * t-test
   *
   * #PRE:  n: number of parameters
   *        theta: parameters
   *        I: matrix having parameters variances on its diagonal
   *        alpha: level used for the test
   * @param t is a vector of size n on input, and contains the
   *        t-statistics on output
   *
   * @return the percentile used for the test.
   */
  double amlet_test_t(int n, double *theta, double **I,
		      double alpha, double *t);

#ifdef __cplusplus
}
#endif

#endif  /* __AMLET_ANALYSIS_H__ */
