/**
 * amlet_data.c
 *
 * AMLET (Another Mixed Logit Estimation Tool)
 * Copyright (C) 2002-2004 Fabian Bastin <bastin@iro.umontreal.ca>
 *
 * Licensed under the Open Software License version 2.1
 */

#include "amlet_data.h"

#include <amlet/amlet.h>

#include "info.h"
#include "Train96.h"
#include "Train99.h"

#include <stdio.h>

#include <oratio/ctools.h>

/**
 * Check that the number of observations is coherent.
 *
 * @param np is number of individuals.
 */
int amlet_check_repetitions(driver *dr, int np, int nobs, int *rep)
{
  int i, j;

  j = rep[0] = 0;
  for (i = 1; i <= np; i++) j += rep[i];
  if (j != nobs) {
    fprintf(stderr, "** FATAL ERROR **: number of observations does not correspond to sum of repetitions.\n");
    driver_clean(dr);
    return AMLET_ERROR_REPETITIONS;
  }

  return 0;
}

/**
 * Data input
 */

/**
 * Read observations from an external file.
 *
 * @param format is a valid data file format identifier.
 */
int import(int format, char *obsname, double ***obs,
	   int **availability, int *choices,
	   int np, int nind, int nalt, int nobs,
	   int idcensor, int iddep, int *times)
{
  switch (format) {
  case TRAIN96:
    return import_train_1996(obsname, obs, availability, choices,
			     np, nind, nalt, nobs, idcensor, iddep);
    break;
  case TRAIN99:
    return import_train_1999(obsname, obs, availability, choices,
			     np, nind, nalt, nobs, idcensor, iddep, times);
    break;
  default:
    return AMLET_ERROR_UNKNOWN_FORMAT;
    break;
  }
}

/**
 * Export observation in an external file.
 *
 * @param format is a valid data file format identifier.
 */
int export(int format, char *obsname, double ***obs,
	   int **availability, const int *choices,
	   int np, int nind, int nalt, int nobs,
	   int idcensor, int *times)
{
  switch (format) {
  case TRAIN96:
    return export_train_1996(obsname, obs, availability,
			     choices, np, nind, nalt, nobs,
			     idcensor);
    break;
  case TRAIN99:
    return export_train_1999(obsname, obs, availability,
			     choices, np, nind, nalt, nobs,
			     idcensor, times);
    break;
  default:
    return AMLET_ERROR_UNKNOWN_FORMAT;
    break;
  }
}
