/**
 * amlet_data.h
 *
 * AMLET
 * Copyright (C) 2002-2008 Fabian Bastin <bastin@iro.umontreal.ca>
 */

#ifndef __AMLET_DATA_H__
#define __AMLET_DATA_H__

/**
 * Data import and export
 */

#include <oratio/ctools.h>

#include "driver.h"

enum {
  TRAIN96 = 1,
  TRAIN99,
  AMLET,
  END_DATA_FORMAT
};

static const c_symbol formats[] = {
  {"Train96", TRAIN96},
  {"Train99", TRAIN99},
  {"amlet", AMLET}
};

#ifdef __cplusplus
extern "C" {
#endif

  /**
   * Check that the number of observations is coherent.
   *
   * @param np is number of individuals.
   */
  int amlet_check_repetitions(driver *dr, int np, int nobs, int *rep);

  /**
   * Read observations from an external file.
   *
   * @param format is a valid data file format identifier.
   */
  int import(int format, char *obsname, double ***obs,
	     int **availability, int *choices,
	     int np, int nind, int nalt, int nobs,
	     int idcensor, int iddep, int *times);

  /**
   * Export observation in an external file.
   *
   * @param format is a valid data file format identifier.
   */
  int export(int format, char *obsname, double ***obs,
	     int **availability, const int *choices,
	     int np, int nind, int nalt, int nobs,
	     int idcensor, int *times);

#ifdef __cplusplus
}
#endif

#endif  /* __AMLET_DATA_H__ */
