/**
 * driver.h
 *
 * AMLET (Another Mixed Logit Estimation Tool)
 * Copyright (C) 2002-2010 Fabian Bastin <bastin@iro.umontreal.ca>
 *
 * Licensed under the Open Software License version 2.1
 */

#ifndef __AMLET_DRIVER_H__
#define __AMLET_DRIVER_H__

#include <stdio.h>
#include <string.h>

#include <amlet/amlet.h>
#include "amlet_config.h"

#include <amlet/dcl.h>
#include <oratio/oratio.h>

#ifdef __cplusplus
extern "C" {
#endif

  /* Lexical analyzer functions */
  void yyinit(FILE *f);

  void driver_init_defaults(driver *dr);

  void driver_post_parsing(driver *dr, dcl_data *md);

  void driver_clean(driver *dr);

  /**
   * Check the parameters defined in the driver
   *
   * @param dr is a pointer to the driver structure.
   * @param data is a pointer to the structure containing important data about
   *   the problem under consideration.
   */
  void driver_check_parameters(driver *dr, amlet_config *ac, dcl_data *data);

  int driver_check_array_pos(driver *dr, int id, int pos);

  /**
   * Do some checks after the parsing
   */
  void driver_check(driver *dr);

  /***************************************************************************/

  void driver_set_generation_process(driver *dr, char *obsname, int nalt);

  /***************************************************************************/

  /**
   * @param x is a double vector of size at least equal to array->n.
   */
  void driver_affect_real(double *x, driver_array *array);
  void driver_affect_integer(int *x, driver_array *array);

  void driver_free_array(driver *dr, int id);

  void *driver_array_address(driver *dr, int id, int pos, int *type);

  void driver_array_action(driver *dr, void *address, int n);

  void *driver_get_address(int id);

  /***************************************************************************/

  void driver_identifier_action(driver *dr, int id);

#ifdef __cplusplus
}
#endif

#endif  /* __AMLET_DRIVER_H__ */

